#!/usr/bin/env python
import os
import pandas as pd
import sys


def main(csvs):
    basename = os.path.commonprefix(csvs)

    # First run once without adjusting duration to save evs
    runs_df = read_csvs(csvs, adjust_duration=False)
    save_evs(runs_df, basename + '_Model2_EVs')
    runs_df.loc[runs_df['condition'].isin(['lowGuess', 'highGuess']),
                'condition'] = 'guess'
    save_evs(runs_df, basename + '_Model1_EVs')


    # # Then run again to create design files
    runs_df = read_csvs(csvs, adjust_duration=True)

    # Model 2 - Split Regressors (as read in from _wide.csv)
    runs_df.to_csv(basename + '_Model2_GuessSplit.csv', index=False)

    # Model 1 - Combine Guessing
    runs_df.loc[runs_df['condition'].isin(['lowGuess', 'highGuess']),
                'condition'] = 'guess'
    runs_df.to_csv(basename + '_Model1_GuessCombined.csv', index=False)
    save_evs(runs_df, basename + '_Model1_EVs')
    runs_df


def read_csvs(csvs, adjust_duration=False):
    runs = []
    for i, csv in enumerate(csvs):
        run_df = read_csv(csv, adjust_duration)
        # run_df['run'] = i + 1  # Should be included in df from log itself
        runs.append(run_df)

    runs_df = pd.concat(runs, ignore_index=True)
    runs_df.sort_values(['run', 'onset'], inplace=True)
    return runs_df


def read_csv(csv, adjust_duration=False):
    df = pd.read_csv(csv)
    onsets_list = []
    column_info = [
        ('highCue', 'cueStartTime', {'valueCondition': 'high'}),
        ('lowCue', 'cueStartTime', {'valueCondition': 'low'}),
        ('highGuess', 'guessStartTime', {'valueCondition': 'high'}),
        ('lowGuess', 'guessStartTime', {'valueCondition': 'low'}),
        ('lowWinFeedback', 'feedbackStartTime', {'trialCondition': 'lowWin',
                                                 }),
        ('lowLoseFeedback', 'feedbackStartTime', {'trialCondition': 'lowLose'
                                                  }),
        ('highWinFeedback', 'feedbackStartTime', {'trialCondition': 'highWin',
                                                  }),
        ('highLoseFeedback', 'feedbackStartTime', {'trialCondition': 'highLose'
                                                   }),
    ]

    for label, onset_col, conditions in column_info:
        onset = df.loc[(~df[onset_col].isnull())]
        for filter_col, filter_val in conditions.iteritems():
            onset = onset.loc[onset[filter_col] == filter_val]

        onset_df = pd.DataFrame({'onset': onset[onset_col].tolist(),
                                 'condition': label,
                                 'duration': [0] * len(onset),
                                 'run': onset['run']})
        onsets_list.append(onset_df)
    run_df = pd.concat(onsets_list, ignore_index=True)

    # Adjust Subsequent Run Timings.
    # I thought Moss did this if 'run' was included, but do it manually here.
    if adjust_duration:
        runDuration = df.loc[~df['runEndTime'].isnull(), 'runEndTime'].iloc[0]
        run_df['onset'] += (run_df['run'] - 1) * runDuration

    return run_df


def save_evs(df, out_prefix):
    for label, cond_df in df.groupby(['condition', 'run']):
        out_file = '%s_%s.EV' % (out_prefix, '%s_run%s' % label)
        cond_df['onset'].to_csv(out_file, index=False, float_format='%.3f')


if __name__ == '__main__':
    main(sys.argv[1:])
    # main(['GUESSING__run1_2017-02-21_001025_wide.csv',
    #       'GUESSING__run2_2017-02-21_001418_wide.csv'])
